<?php
/**
 * @file
 * This file holds every menu generated by this module
 */


module_load_include('inc', 'domain_rights_management', 'includes/helper_functions');

/**
 * The main form for the local administrator.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_mainMenu()
 * @return string
 *   HTML-Markup of administrative menu für this module.
 */
function domain_rights_management_mainMenu() {
  $ret     = '';
  $ret     = '<ul class= "action-links"><li>';
  $ret    .= l(t('Add user'), 'admin/useradministration/CreateUser');
  $ret    .= '</li></ul>';

  $tmp     = domain_get_domain();
  $users   = domain_rights_management_list_users($tmp['domain_id']);

  $do      = domain_get_domain();
  $header  = array(t('Username'),
    t('Role'), t('Status'), t('Last access'), t('Operations'));

  $content = array();

  foreach ($users as $user => $array) {
    $rows      = array();
    $rows[]    = $user;

    if (count($array['role']) == 0) {
      $rows[] = '';
    }
    else {
      $rows[] = theme_item_list(
        array(
          'items' => $array['role'],
          'type' => 'ul',
          'title' => '',
          'attributes' => array()));
    }

    if ($array['status'] == 1) {
      $rows[]  = t('active');
    }
    else {
      $rows[]  = t('inactive');
    }

    if ($array['access'] == 0) {
      $rows[]  = t('Never');
    }
    else {
      $rows[]  = date('d-m-Y H:i', $array['access']);
    }

    $links     = '';
    $links    .= l(t('edit'), 'admin/useradministration/AlterUser/' . $array['uid']) . ' ';
    $links    .= l(t('delete'), 'admin/useradministration/delete/' . $array['uid']) . ' ';

    if (user_is_blocked(user_load($array['uid'])->name)) {
      $links  .= l(t('unblock'), 'admin/useradministration/unblock/' . $array['uid']);
    }
    else {
      $links  .= l(t('block'), 'admin/useradministration/block/' . $array['uid']);
    }
    $rows[]    = $links;
    $content[] = $rows;
  }
  // Reference http://drupal.org/node/156863
  $ret .= theme('table', array('header' => $header, 'rows' => $content));
  return $ret;
}


/**
 * Form for the global administrator.
 *
 * @author cupcakemuncher
 * @name domain_rights_management__globaladmin_formgenerator
 * 
 * @param string $form_id
 *   The ID of the form
 *
 * @return array()
 *   An array describing the form to be displayed.
 */
function domain_rights_management_globaladmin_formgenerator($form_id) {
  global $base_path;
  $form                      = array();

  $roles                     = user_roles(TRUE);
  $localdomains              = domain_rights_management_query_local_domains();
  $activated_roles_by_domain = array();
  $form['general']           = array(
    '#type'        => 'fieldset',
    '#title'       => t('General settings'),
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
  );

  $form['general']['allowed_roles_general'] = array(
    '#type'          => 'checkboxes',
    '#options'       => $roles,
    '#default_value' => domain_rights_management_list_general_rights() ,
  );
  $form['general']['overwrite'] = array(
    '#type' => 'submit',
    '#value' => t('Overwrite'),
  );

  $form['specific'] = array(
    '#type'  => 'fieldset',
    '#title' => t('Specific settings'),
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
  );
  $form['general']['overwrite_all'] = array(
    '#type'     => 'checkbox',
    '#title'    => t('Overwrite existing configurations as well?'));

  foreach ($localdomains as $id => $namearray) {
    $form['specific'][$namearray['machinename']] = array(
      '#type'   => 'fieldset',
      '#title'  => t($namearray['domainname']),
      '#collapsible' => TRUE,
      '#collapsed'   => TRUE,
    );
    $activated_roles = domain_rights_management_list_roles_for_domain($id);
    $tmp = array();
    foreach ($activated_roles as $rid => $name) {
      $tmp[] = $rid;
    }
    $form['specific'][$namearray['machinename']]['id_' . $namearray['machinename']] = array(
      '#type'  => 'hidden',
      '#value' => $id,
    );
    $form['specific'][$namearray['machinename']]['allowed_roles_' . $namearray['machinename']] = array(
      '#type'          => 'checkboxes',
      '#options'       => $roles,
      '#default_value' => $tmp ,
    );
  }

  $form['specific']['Exceptions per User'] = array(
    '#type'         => 'fieldset',
    '#title'        => t('Exceptions per user'),
    '#collapsible'  => TRUE,
    '#collapsed'    => TRUE,
  );

  $localadmins = domain_rights_management_query_local_admins();

  foreach ($localadmins as $uid => $name) {
    $form['specific']['Exceptions per User'][$name] = array(
      '#type'        => 'fieldset',
      '#title'       => t($name),
      '#collapsible' => TRUE,
      '#collapsed'   => TRUE,
    );
    $form['specific']['Exceptions per User'][$name]['id_' . $name] = array(
      '#type'  => 'hidden',
      '#value' => $id,
    );

    $tmp = domain_rights_management_query_exceptions_for_user($uid);
    $t = array();
    foreach ($tmp as $exceptionid => $exceptionname) {
      $t[] = $exceptionid;
    }

    $form['specific']['Exceptions per User'][$name]['roles_' . $name] = array(
      '#type'          => 'checkboxes',
      '#options'       => $roles,
      '#default_value' => $t,
    );
  }
  $form['specific']['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}


/**
 * Submit-function for the global-administrator form.
 *
 * @author cupcakemuncher
 * @name domain_rights_management__globaladmin_formgenerator_submit
 *
 * @param array $form
 *   The submitted form.
 * @param array $form_state
 *   The state of the submitted form.
 *
 * @see http://drupal.org/node/751826
 */
function domain_rights_management_globaladmin_formgenerator_submit($form, &$form_state) {
  if ($form_state['values']['op'] == 'Overwrite') {
    domain_rights_management_insert_delete_general_rights($form_state['values']['allowed_roles_general']);
    $query  = db_select('domain', 'd');
    $query->addField('d', 'domain_id');
    $res    = $query->execute();
    $result = TRUE;

    if ($form_state['values']['overwrite_all'] == 1) {
      domain_rights_management_flush_rights_domain();
      foreach ($res as $entry) {
        try{
          domain_rights_management_insert_delete_privilege($entry->domain_id, $form_state['values']['allowed_roles_general']);
        }
        catch (Exception $r) {
          $result = FALSE;
          break;
        }
      }
    }

    if ($result) {
      drupal_set_message(t('Your submission has been saved.'));
    }
    else {
      form_set_error('title', 'There has been an error in the Database please contact your Administrator.');
    }
  }
  else {
    $localdomains = domain_rights_management_query_local_domains();
    $localadmins  = domain_rights_management_query_local_admins();
    try{
      foreach ($localdomains as $id => $namearray) {
        domain_rights_management_insert_delete_privilege($form_state['values']['id_' . $namearray['machinename']], $form_state['values']['allowed_roles_' . $namearray['machinename']]);
      }
      foreach ($localadmins as $uid => $adminname) {
        domain_rights_management_insert_delete_exceptions_for_user($uid, $form_state['values']['roles_' . $adminname]);
      }
      drupal_set_message(t('Your submission has been saved.'));
    }
    catch (Exception $r) {
      form_set_error('title', t('There has been an error in the Database please contact your Administrator.'));
    }
  }
}


/**
 * Delete form.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_deleteConfirmation_form
 *
 * @param array() $form
 *   The form.
 * @param array() $form_state
 *   The state of the form.
 * @param stdClass $acc
 *   The account in question.
 *
 * @return array()
 *   The form-array to be displayed.
 */
function domain_rights_management_deleteConfirmation_form($form, $form_state, $acc) {
  if ($acc == FALSE) {
    // There is no userobject with the uid passed in the url.
    return $form;
  }

  $form['uid']    = array('#type' => 'hidden', '#value' => $acc->uid);
  $form['cancel'] = array('#type' => 'submit', '#value' => 'cancel');
  $form['submit'] = array('#type' => 'submit', '#value' => 'submit');
  return $form;
}

/**
 * Receives the submission of the form to confirm the deletion of a user.
 * 
 * @author cupcakemuncher
 * @name domain_rights_management_delete_confirmation_cancel_confirm_form_submit
 * 
 * @param array $form
 *   The form.
 *   
 * @param array $form_state
 *   The state of the form.
 */
function domain_rights_management_delete_confirmation_cancel_confirm_form_submit($form, $form_state) {
  if (module_load_include('inc', 'user', 'user.pages') == FALSE) {
    drupal_set_message(t("A Problem occured. File couldn't be loaded."), 'error');
    drupal_goto();
  }
  drupal_save_session(FALSE);
  global $user;
  $current_user = $user;
  $user         = user_load(1);

  user_cancel_confirm_form_submit($form, $form_state);

  $user                   = $current_user;
  $form_state['redirect'] = 'admin/useradministration';
}

/**
 * Validates if given uid is identifiying a user assigned to current domain.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_deleteConfirmation_form_validate
 *
 * @param array $elements
 *   The formarray to validate.
 *   
 * @param array $form_state
 *   The state of the form.
 *   
 * @param String $form_id
 *   The name of the form.
 */
function domain_rights_management_deleteConfirmation_form_validate(&$elements, &$form_state, $form_id = NULL) {
  if (($form_state['values']['op'] != 'cancel') && ($form_state['values']['op'] != 'submit')) {
    form_set_error('title', t('Unknown operation'));
  }
  if (!domain_rights_management_is_user_assigned_to_current_domain($form_state['values']['uid'])) {
    form_set_error('title', t('User-ID or Domain unappropriate'));
  }
}

/**
 * Deletes the user.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_deleteConfirmation_form_submit
 *
 * @param array $form
 *   The form-array of the submitted form.
 *   
 * @param array $form_state
 *   The state of the submitted form.
 */
function domain_rights_management_deleteConfirmation_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] == 'cancel') {
    drupal_set_message(t('Operation canceled.'));
  }
  else {
    try{
      if (domain_rights_management_delete_user($form_state['values']['uid'])) {
        drupal_set_message(t('User deleted.'));
      }
      else {
        form_set_error('title', t('You do not have the rights to perform this action.Try logging into your appropriate domain.'));
      }
    }
    catch (Exception $e) {
      form_set_error('title', t('There has been a Databaseerror please report it to your administrator.'));
    }
  }
  $form_state['redirect'] = array('admin/useradministration');
}

/**
 * Generates the form for a dialog to confirm the blocking of a user.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_blockConfirmation_form
 *
 * @param array $form
 *   The form-array to be displayed.
 *   
 * @param array $form_state
 *   The state of the form.
 *   
 * @param stdClass $account
 *   The useraccount in question.
 *
 * @return array()
 *   Form-array.
 */
function domain_rights_management_blockConfirmation_form($form, $form_state, $account) {
  if ($account == FALSE) {
    return $form;
  }
  $form['uid']    = array('#type' => 'hidden', '#value' => $account->uid);
  $form['cancel'] = array('#type' => 'submit', '#value' => 'cancel');
  $form['submit'] = array('#type' => 'submit', '#value' => 'submit');
  return $form;
}

/**
 * Validates the blockConfirmation-dialog.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_blockConfirmation_form_validate
 *
 * @param array $elements
 *   The formarray.
 *   
 * @param array $form_state
 *   The state of the formarray.
 *   
 * @param String $form_id
 *   The name of the form to be validated.
 */
function domain_rights_management_blockConfirmation_form_validate(&$elements, &$form_state, $form_id = NULL) {
  if (($form_state['values']['op'] != 'cancel') && ($form_state['values']['op'] != 'submit')) {
    form_set_error('title', t('Operation unkown.'));
  }
  if (!user_load($form_state['values']['uid']) ||
      !domain_rights_management_is_user_assigned_to_current_domain($form_state['values']['uid'])) {
    form_set_error('title', t('User or Domain invalid.'));
  }
}

/**
 * Allows local admin to block chosen user.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_blockConfirmation_form_submit
 *
 * @param array $form
 *   The submitted form.
 *   
 * @param array $form_state
 *   The state of the submitted form.
 */
function domain_rights_management_blockConfirmation_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] == 'cancel') {
    drupal_set_message(t('Operation canceled.'));
  }
  else {
    if (!domain_rights_management_block_user($form_state['values']['uid'])) {
      form_set_error('title', t('You cannot block root or yourself.'));
    }
    else {
      drupal_set_message(t('User blocked.'));
    }
  }
  $form_state['redirect'] = 'admin/useradministration';
}

/**
 * Generates form about unblocking a certain user.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_unblockConfirmation_form
 *
 * @param array $form
 *   The from to be displayed to the user
 *   
 * @param array $form_state
 *   The state of the form.
 *   
 * @param stdClass $account
 *   The useraccount in question
 *
 * @return array()
 *   Formarray.
 */
function domain_rights_management_unblockConfirmation_form($form, $form_state, $account) {
  // There is no userobject with the uid passed in the url.
  if ($account == FALSE) {
    return $form;
  }
  $form['uid']    = array('#type' => 'hidden', '#value' => $account->uid);
  $form['cancel'] = array('#type' => 'submit', '#value' => 'cancel');
  $form['submit'] = array('#type' => 'submit', '#value' => 'Submit');
  return $form;
}

/**
 * This function allows the local admin to unblock a chosen user.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_unblockConfirmation_form_submit
 *
 * @param array() $form
 *   The submitted form.
 * @param array() $form_state
 *   The state of the submitted form.
 */
function domain_rights_management_unblockConfirmation_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] == 'cancel') {
    drupal_set_message(t('Operation canceled'));
  }
  else {
    if (!domain_rights_management_unblock_user($form_state['values']['uid'])) {
      form_set_error('title', t('You do not have sufficiant rights.'));
    }
    else {
      drupal_set_message(t('User unblocked.'));
    }
  }
  $form_state['redirect'] = 'admin/useradministration';
}

/**
 * Validates the form.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_unblockConfirmation_form_validate
 *
 * @param array $form
 *   The form in question.
 * @param array() $form_state
 *   The state of the form in question.
 * @param string $form_id
 *   The name of the form.
 */
function domain_rights_management_unblockConfirmation_form_validate(&$form, &$form_state, $form_id = NULL) {
  if (($form_state['values']['op'] != 'cancel') && ($form_state['values']['op'] != 'Submit')) {
    form_set_error('title', t('Operation unkown'));
  }
  if (!user_load($form_state['values']['uid']) ||
      !domain_rights_management_is_user_assigned_to_current_domain($form_state['values']['uid'])) {
    form_set_error('title', t('User on invalid domain'));
  }
};

/**
 * Creates form for user-creationg.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_create_user_form_generator
 *
 * @param array $form
 *   The form in question.
 * @param array $form_state
 *   The state of the form in question.
 */
function domain_rights_management_create_user_form_generator($form, $form_state) {
  // Manipulating form not possible since
  // the user-module denies usage of forms to non-admin-users
  if (module_load_include('inc', 'user', 'user.pages') == FALSE) {
    drupal_set_message(t("A Problem occured. File couldn't be loaded."), 'error');
    drupal_goto();
  }

  global $user;
  // The following lines elivates the current user to uid 1,
  // this should have no effect
  // Since we only extract the form we need for creating a user.
  $original_user = $user;
  // See http://drupal.org/node/218104
  drupal_save_session(FALSE);
  $user = user_load(1);
  // Extracting form as user->uid== 1
  $form = user_register_form($form, $form_state);
  // $form = drupal_get_form('user_register_form');
  // This way we will prepare the form but still
  // have the chance to execute the #after_build function.
  $form['#after_build'][] = 'domain_rights_management_create_user_alter';
  drupal_prepare_form('user_register_form', $form, $form_state);

  // User_account_form($form,$form_state) .
  // Assigning original user-object back to user .
  // This is important because altough drupal_save_session(FALSE)
  // is called the $userobject is saved after request is served .
  $user = $original_user;

  // Inspired by and taken from
  // http://erikwebb.net/blog/drupal-design-patterns/hooks .
  // @see user_module_invoke() .
  return $form;
}

/**
 * This function assignes the created user to the current domain.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_create_user_form_submit
 *
 * @param array() $form
 *   The form-array in question.
 * @param array() $form_state
 *   The state of the form.
 */
function domain_rights_management_create_user_form_submit($form, $form_state) {
  $dom = domain_get_domain();
  db_insert('domain_editor')
  ->fields(array('uid', 'domain_id'), array($form['#user']->uid, $dom['domain_id']))
  ->execute();
}

/**
 * Calls formbuilders for alteruser form. Requires elevated permission.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_alter_user_form
 *
 * @param array $form
 *   The form-array in question.
 * @param array() $form_state
 *   The array describing the state of the form-
 * @param stdClass $user_obj
 *   Object representing the user in question.
 */
function domain_rights_management_alter_user_form($form, &$form_state, $user_obj) {
  if (module_load_include('inc', 'user', 'user.pages') == FALSE) {
    drupal_set_message(t("A Problem occured. File couldn't be loaded."), 'error');
  }
  else {
    if (($user_obj != FALSE) && domain_rights_management_is_user_assigned_to_current_domain($user_obj->uid)) {
      global $user;

      // The following lines elivates the current user to uid 1,
      // this should have no effect
      // since we only extract the form we need for creating a user.
      // See http://drupal.org/node/218104 .
      drupal_save_session(FALSE);
      $original_user          = $user;
      $user                   = user_load(1);
      // Extracting form as user->uid== 1.
      $form                   = user_profile_form($form, $form_state, $user_obj);
      // Assigning original user-object back to user.
      // This is important because altough drupal_save_session(FALSE)
      // is called the $userobject is saved after request is served.
      // We also need access to the roles.
      $user                   = $original_user;
      // Runs after the form is readily build and
      // cuts out all the roles the user is not allowed to grant.
      $form['#after_build'][] = 'domain_rights_management_alter_user_alter';
      // Overwriting redirect.
      $form_state['redirect'] = 'admin/useradministration';
      drupal_prepare_form('user_profile_form', $form, $form_state);
      return $form;
    }
    else {
      drupal_set_message(t('Please log into your domain.'), 'warning');
    }
  }
  // Visiting front-page
  drupal_goto();
}

/**
 * Filters all roles the user is not allowed to edit.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_create_user_alter
 *
 * @param array() $form
 *   The form-array to be altered.
 *   
 * @param array() $form_state
 *   State of the form-array.
 *   
 * @return array()
 *   Altered form-array.
 */
function domain_rights_management_create_user_alter(&$form, &$form_state) {
  // TODO replace unset with hide() and test for sec.
  unset($form['#user']->domain_user);
  unset($form['account']['signature_settings']['signature']['format']['#groups']['domain_user']);
  unset($form['account']['signature_settings']['#groups']['domain_user']);
  unset($form['#fieldgroups']['#group_children']['domain']);
  unset($form['domain_user']);
  unset($form['#group_children']['domain']);
  unset($form['locale']['#groups']['domain_user']);

  global $user;
  $tmp                                  = domain_get_domain();
  $allowed_roles                        = domain_rights_management_list_roles_for_domain($tmp['domain_id']);
  $allowed_roles                       += domain_rights_management_query_exceptions_for_user($user->uid);
  $form['account']['roles']['#options'] = $allowed_roles;
  $elements                             = array_values($allowed_roles);

  // Submitfunction an be used in addition nothing has to be called explicitly.
  $form['#submit'][]                    = 'domain_rights_management_create_user_form_submit';
  // Redirect overwritten otherwise we will end up at admin/people/create
  $form_state['redirect']               = 'admin/useradministration';

  foreach ($form['account']['roles'] as $key => $value) {
    if (isset($value['#title'])) {
      if (!in_array($value['#title'], $elements) && ($value['#theme'] == 'checkbox')) {
        unset($form['account']['roles'][$key]);
      }
    }
  }

  foreach (module_implements('domain_rights_management_create_user') as $module) {
    $functionname = $module . '_domain_rights_management_create_user';
    // Will call all modules implementing hook_hook_name
    // and can pass each argument as reference determined
    // by the function declaration.
    $form         = $functionname($form, $form_state);
  }
  return $form;
}

/**
 * This function assignes the created user to the current domain.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_alter_user_form_submit
 *
 * @param array() $form
 *   The form-array of the submitted form.
 * @param array() $form_state
 *   The state of the submitted form-array.
 */
function domain_rights_management_alter_user_form_submit($form, &$form_state) {
  global $user;
  $current_user = $user;
  drupal_save_session(FALSE);
  $user         = user_load(1);
  user_profile_form_submit($form, $form_state);
  $user         = $current_user;

  if (user_access('assign domain editors')) {
    $dom = domain_get_domain();
    db_insert('domain_editor')
    ->fields(array('uid', 'domain_id'), array($form['#user']->uid, $dom['domain_id']))
    ->execute();
  }
}

/**
 * Filters all roles the user is not allowed to edit.
 *
 * @author cupcakemuncher
 * @name domain_rights_management_create_user_alter
 * 
 * @param array() $form
 *   The form-array to alter.
 * @param array() $form_state
 *   The state of the form-array
 *
 * @return array()
 *   The altered form-array.
 */
function domain_rights_management_alter_user_alter(&$form, &$form_state) {
  unset($form['#user']->domain_user);
  unset($form['account']['signature_settings']['signature']['format']['#groups']['domain_user']);
  unset($form['account']['signature_settings']['#groups']['domain_user']);
  unset($form['#fieldgroups']['#group_children']['domain']);
  unset($form['domain_user']);
  unset($form['#group_children']['domain']);
  unset($form['locale']['#groups']['domain_user']);

  global $user;
  $tmp                                  = domain_get_domain();
  $allowed_roles                        = domain_rights_management_list_roles_for_domain($tmp['domain_id']);
  $allowed_roles                       += domain_rights_management_query_exceptions_for_user($user->uid);
  $form['account']['roles']['#options'] = $allowed_roles;
  $elements                             = array_values($allowed_roles);
  $form_state['redirect']               = 'admin/useradministration';

  foreach ($form['account']['roles'] as $key => $value) {
    if (isset($value['#title'])) {
      if (!in_array($value['#title'], $elements) && ($value['#theme'] == 'checkbox')) {
        unset($form['account']['roles'][$key]);
      }
    }
  }

  foreach (module_implements('domain_rights_management_alter_user') as $module) {
    $functionname = $module . '_domain_rights_management_alter_user';
    $form  = $functionname($form, $form_state);
  }
  $form['#submit'] = array('domain_rights_management_alter_user_form_submit');
  unset($form['actions']['cancel']);
  return $form;
}
